@@ -108,7 +108,7 @@ typedef struct _rfbScreenInfo
 
     rfbPixelFormat rfbServerFormat;
     rfbColourMap colourMap; /* set this if rfbServerFormat.trueColour==FALSE */
-    const char* desktopName;
+    char* desktopName;
     char rfbThisHost[255];
 
     rfbBool autoPort;
@@ -582,6 +582,8 @@ extern void rfbInitServer(rfbScreenInfoPtr rfbScreen);
 extern void rfbNewFramebuffer(rfbScreenInfoPtr rfbScreen,char *framebuffer,
  int width,int height);
 
+extern void rfbSetDesktopName(rfbScreenInfoPtr rfbScreen, const char *name);
+
 extern void rfbScreenCleanup(rfbScreenInfoPtr screenInfo);
 
 /* call one of these two functions to service the vnc clients.
